import { InitWasm } from './wasm'
import { addTs, calculatePiLeibnizTs, countPrimesTs } from './benchmark'

import { performance } from 'perf_hooks'
/**
 * Runs a function, measures its execution time, and prints the result.
 * @param name Name of the benchmark.
 * @param func The function to execute.
 * @param args Arguments to pass to the function.
 */
function runBenchmark(
    name: string,
    func: (...args: any[]) => any,
    ...args: any[]
) {
    console.log(`\nRunning benchmark: ${name}`)
    console.log(`Arguments: ${args.join(', ')}`)

    const startTime = performance.now()
    const result = func(...args)
    const endTime = performance.now()
    const duration = endTime - startTime

    console.log(`Result: ${result}`)
    console.log(`Execution Time: ${duration.toFixed(3)} ms`)
    return duration
}

// --- Benchmark Parameters ---
const PI_ITERATIONS = 100_000_000_000 // High number for Pi calculation
const PRIME_LIMIT = 100_000_000 // Limit for prime counting
// --- End Benchmark Parameters ---

InitWasm().then(wasm => {
    // Verify AS module loaded
    const sumAs = wasm.addAs(5, 7)
    console.log(`AS Module Loaded Verification (addAs(5, 7)): ${sumAs}`)
    if (sumAs !== 12) {
        console.warn('AS addAs function did not return expected result!')
    }
    const sumTs = addTs(5, 7)
    console.log(`TS Implementation Verification (addTs(5, 7)): ${sumTs}`)

    // --- Run Benchmarks ---

    // Benchmark: Calculate Pi
    runBenchmark(
        'TypeScript: Calculate Pi (Leibniz)',
        calculatePiLeibnizTs,
        PI_ITERATIONS,
    )
    runBenchmark(
        'AssemblyScript: Calculate Pi (Leibniz)',
        wasm.calculatePiLeibnizAs,
        PI_ITERATIONS,
    )

    // Benchmark: Count Primes
    runBenchmark(
        'TypeScript: Count Primes (Trial Division)',
        countPrimesTs,
        PRIME_LIMIT,
    )
    runBenchmark(
        'AssemblyScript: Count Primes (Trial Division)',
        wasm.countPrimesAs,
        PRIME_LIMIT,
    )

    console.log('\nBenchmark finished.')
})
